--[[
AdiBags_BaublewormJunk - Adds filters to AdiBags for the Junk items needed to purchase the Baubleworms in Oribos.
Copyright 2021 Protuhj
All rights reserved.
--]]

local _, ns = ...

local addon = LibStub('AceAddon-3.0'):GetAddon('AdiBags')
local L = setmetatable({}, {__index = addon.L})

-- The filter itself
local FILTER_NAME = "Baubleworms Junk"
local setFilter = addon:RegisterFilter(FILTER_NAME, 100, 'ABEvent-1.0')
setFilter.uiName = FILTER_NAME
setFilter.uiDesc = L["Adds filters to AdiBags for the Junk items needed to purchase the Baubleworms in Oribos.|n\
If you use item locking:|n \
      - To be able to sell duplicates or mail items to alts, you'll need to toggle off item locking.|n\
      - You don't need to disable item locking to buy the pets.|n\
      - Important: Once you get all the pets and want to remove this addon, disable item locking so you can get rid of the items!"]

function setFilter:OnInitialize()
  self.db = addon.db:RegisterNamespace(FILTER_NAME, {
    profile = { enable = true, lock = true},
    char = {  },
  })
end

function setFilter:Update()
  self:SendMessage('AdiBags_FiltersChanged')
end

function setFilter:OnEnable()
  addon:UpdateFilters()
end

function setFilter:OnDisable()
  addon:UpdateFilters()
end

local JUNK_ITEMS = {
    -- Ruby Baubleworm
    [36812] = true,     -- Ground Gear
    [62072] = true,     -- Robble's Wobbly Staff
    [67410] = true,     -- Very Unlucky Rock
    -- Topaz Baubleworm
    [11406] = true,     -- Rotting Bear Carcass
    [11944] = true,     -- Dark Iron Baby Booties
    [25402] = true,     -- The Stoppage Force
    -- Turquoise Baubleworm
    [3300] = true,     -- Rabbit's Foot
    [3670] = true,     -- Large Slimy Bone
    [6150] = true,     -- A Frayed Knot
    -- TODO DEBUG 
    -- [176853] = true,     -- Spindly Flesh
}

function setFilter:Filter(slotData)
  if not self.db.profile.enable or not JUNK_ITEMS[slotData.itemId] then
    return nil
  end
  local item = Item:CreateFromBagAndSlot(slotData.bag, slotData.slot)
  if self.db.profile.lock then
    if not item:IsItemLocked() then
        --print(slotData.name .. " Item not locked, locking it")
        item:LockItem()
    end
  else
    if item:IsItemLocked() then
        --print(slotData.name .. " Item locked, unlocking it")
        item:UnlockItem()
    end
  end
  return FILTER_NAME
end

function setFilter:GetOptions()
  return {
    enable = {
      name = L['Enable'],
      desc = L['Check this if you want a section for the Junk currency for the Baubleworms you can buy in Oribos.'],
      type = 'toggle',
      order = 10,
    },
    lock = {
      name = L['Lock currency'],
      desc = L['Should the currency be locked using the WoW API?'],
      type = 'toggle',
      order = 20,
    },
  }, addon:GetOptionHandler(self, false, function() return self:Update() end)
end
